/**
 * Car Courtesy Light
 *
 * Version: 1.0
 * Date: 2011-03-20
 * (c) Geert Van Espen (geert@vanespen.com)
 */

//#define DWENGO
#define TIMEOUT_SECONDS 25
#define TIMEOUT TIMEOUT_SECONDS * 1000 / 12
#define SOFT_STOP_DOOR 48
#define SOFT_START_MANUAL 26
#define SOFT_STOP_MANUAL 26

#ifdef __18F4550
#define DWENGO
#endif

#ifdef DWENGO

#include <dwengoConfig.h>
#include <dwengoBoard.h>

#else

#include "CarCourtesyLight.h"
//#include "CarCourtesyLightBoard.h"

#define TRUE    1
#define FALSE   0
#define HIGH    1
#define LOW     0
#define PRESSED 0
#define INPUT   1
#define OUTPUT  0

void delay_ms(int);
void setCursorLCD(unsigned char a, unsigned char b) {}
void appendIntToLCD(int a) {}
void appendStringToLCD_(const far rom char* message) {}
void appendStringToLCD_(const far rom char*);
#define appendStringToLCD(message)   appendStringToLCD_((const far rom char*)(message))
void initBoard(void) {}
void backlightOn(void) {}


#ifdef __18F2550
#include <p18f2550.h>
#endif

#ifdef __18F4550
#include <p18f4550.h>
#endif

#endif

#include <timers.h>
#include <pwm.h>
#include <delays.h>


/*unsigned int g[64] = {
  0, 3, 7, 7, 7, 11, 11, 11, 11, 11, 11, 15, 15, 15, 15, 19, 
  19, 19, 23, 23, 27, 27, 31, 35, 35, 39, 43, 47, 51, 55, 59, 63, 
  71, 75, 83, 91, 99, 107, 115, 127, 139, 151, 163, 179, 195, 211, 231, 251, 
  275, 299, 327, 359, 391, 427, 467, 507, 555, 603, 659, 719, 787, 859, 939, 1023
};*/
unsigned int g[64] = {
  0, 2, 3, 5, 7, 8, 9, 10, 11, 11, 11, 12, 13, 14, 15, 16, 
  17, 19, 23, 23, 27, 27, 31, 35, 35, 39, 43, 47, 51, 55, 59, 63, 
  71, 75, 83, 91, 99, 107, 115, 127, 139, 151, 163, 179, 195, 211, 231, 251, 
  275, 299, 327, 359, 391, 427, 467, 507, 555, 603, 659, 719, 787, 859, 939, 1023
};

#define MAX 63

void main(void) {
  unsigned int p = 0, pt = 0;
  unsigned int mode = 0; // 0 = door mode    1 = manual mode
  int reverse = 0, step = 1;
  int teller = 0, message_dim_set = 0;

  // Initialise the board.
  initBoard();

  // Set internal oscillator to 8 MHz.
  OSCCONbits.IRCF0 = 1;
  OSCCONbits.IRCF1 = 1;
  OSCCONbits.IRCF2 = 1;


  // Show startup info on LCD.
  backlightOn();
  appendStringToLCD("CarCourtesyLight");
  
  // Set inputs/outputs.
  TRISAbits.TRISA4 = 0;
  TRISAbits.TRISA5 = 0;

#ifdef DWENGO
  TRISDbits.TRISD0 = 0;
  TRISDbits.TRISD1 = 0;
  TRISDbits.TRISD2 = 0;
  TRISDbits.TRISD3 = 0;
  TRISDbits.TRISD4 = 0;
  TRISDbits.TRISD5 = 0;
  TRISDbits.TRISD6 = 0;
  TRISDbits.TRISD7 = 0;

  // All LEDs of Dwengo board off.
  PORTD = 0;
#endif

  // Enable pull-ups on PORTB.
  INTCON2bits.NOT_RBPU = 0;

  // Initialise PWM module.
  TRISCbits.TRISC1 = 0;
  TRISCbits.TRISC2 = 0;
  PORTAbits.RA4 = 1;
  
  PORTCbits.RC1 = 0;
  PORTAbits.RA5 = 0;

  OpenPWM2(0xFF);
  OpenTimer2(TIMER_INT_OFF & T2_PS_1_16 & T2_POST_1_1);

  // Set duty cycle of PWM to 0 percent.
  SetDCPWM2(0);

  // Wait a short while.
  delay_ms(12);

  mode = 0;
  if (PORTBbits.RB1 == 0) mode = 1;

  while (mode == 1)
  {
    // MANUAL MODE.

    // In manual mode, there is no need to watch for door closed. Activate block relay to prevent
    // the door relay from buzzing because of the PWM voltage on it.
    PORTAbits.RA5 = 1;

    // Soft start.
    while (p < MAX)
    {
      p++;                                 // Increment pointer to look-up table.
      pt = g[p];                           // Translate pointer into gamma-value.
      SetDCPWM2(pt);                       // Dim LED accordingly.
      delay_ms(SOFT_START_MANUAL);
      setCursorLCD(1, 0);
      appendIntToLCD(PORTBbits.RB0);       // Debug info to LCD.
      appendIntToLCD(PORTBbits.RB1);
      if (PORTBbits.RB1) break;            // If manual switch put to OFF, stop the soft start cycle.
    }
    while (PORTBbits.RB1 == 0)  // Wait until manual switch off.
    {
      setCursorLCD(1, 0);
      appendIntToLCD(PORTBbits.RB0);
      appendIntToLCD(PORTBbits.RB1);
      delay_ms(12);
    }
    while (TRUE)
    {
      p--;
      pt = g[p];
      SetDCPWM2(pt);
      delay_ms(SOFT_STOP_MANUAL);
      while (p == 0)
      {
        PORTAbits.RA5 = 0;
        PORTAbits.RA4 = 0;
      }
      if (PORTBbits.RB1 == 0) break;
      setCursorLCD(1, 0);
      appendIntToLCD(PORTBbits.RB0);
      appendIntToLCD(PORTBbits.RB1);
    }
  }
  
  while (mode == 0)
  {
    // DOOR MODE.
    teller = 0;
    while (PORTBbits.RB0)  // Wait until door closed.
    {
      SetDCPWM2(0);
      setCursorLCD(1, 0);
      appendIntToLCD(PORTBbits.RB0);
      appendIntToLCD(PORTBbits.RB1);
      delay_ms(12);
    }

    // Door closed, set light to 100% duty cycle.
    setCursorLCD(1, 0);
    appendIntToLCD(PORTBbits.RB0);
    appendIntToLCD(PORTBbits.RB1);
    //delay_ms(1);
    SetDCPWM2(1023); // 100% duty cycle.
    PORTAbits.RA5 = 1; // Activate block relay, no more need to monitor the light now.
    while (TRUE)
    {
      if (PORTBbits.RB2 == 0)              // If external trigger is grounded, immediately stop the extend cycle.
        teller = TIMEOUT;

      teller++;   // This is for counting the time. 2083 is about 25 seconds.
      delay_ms(12);
      if (teller >= TIMEOUT)
      {
        if (!message_dim_set)
        {
          appendStringToLCD(" fading out");
          message_dim_set = 1;
        } 
        // Timeout reached, start to softly fade the light.
        p = MAX;
        while (TRUE)
        {
          p--;
          pt = g[p];
          SetDCPWM2(pt);
          delay_ms(SOFT_STOP_DOOR);
          while (p == 0)
          {
            // All done, all relays off and PIC will die here.
            PORTAbits.RA5 = 0;
            PORTAbits.RA4 = 0;
          }
        }
      }
    }
  }


  // Experimental. This is just a dimmer.
  // Buttons: down/up dim down/up.
  /*
  p = 0;
  while (mode == 2)
  {
    PORTAbits.RA5 = 1;
    if (p >= MAX) p = MAX;
    if (p <= 0) p = 0;

    if (SW_N == PRESSED) PORTDbits.RD0 = 1; else PORTDbits.RD0 = 0;
    if (SW_S == PRESSED) PORTDbits.RD1 = 1; else PORTDbits.RD1 = 0;
    if (SW_W == PRESSED) PORTDbits.RD2 = 1; else PORTDbits.RD2 = 0;
    if (SW_E == PRESSED) PORTDbits.RD3 = 1; else PORTDbits.RD3 = 0;
    if (SW_C == PRESSED) PORTDbits.RD4 = 1; else PORTDbits.RD4 = 0;
   
    step = 1;
    if (PORTBbits.RB2 == 0)
    {
      p = 0;
      reverse = 0;
      while (TRUE)
      {
        if (reverse) p--; else p++;
        pt = g[p];
        SetDCPWM2(pt);
        delay_ms(48);
        if (p == MAX) reverse = 1;
        if (p == 0) reverse = 0;
      }
    }

    //if (SW_W == PRESSED && p > 0) p -= step;
    //if (SW_E == PRESSED && p < MAX) p += step;

    if (SW_S == PRESSED && p > 0)
    {
      p--;
      delay_ms(25);
    }
    if (SW_N == PRESSED && p < MAX)
    {
      p++;
      delay_ms(25);
    }

    pt = g[p];

    setCursorLCD(1, 0);
    appendIntToLCD(p);
    appendStringToLCD(" ");
    appendIntToLCD(pt);
    appendStringToLCD(" ");
    appendStringToLCD(" ");
    appendStringToLCD(" ");

    SetDCPWM2(pt);

    delay_ms(12);
  }
  

  // Just testing.
  PORTAbits.RA4 = 1;
  PORTAbits.RA5 = 1;
  delay_ms(100);
  PORTAbits.RA5 = 1;

  TRISDbits.TRISD0 = 0;
  TRISDbits.TRISD6 = 0;
  TRISDbits.TRISD7 = 0;


    while (p < MAX)
    {
      p++;
      pt = g[p];
      SetDCPWM2(pt);
      delay_ms(48);
      setCursorLCD(1, 0);
      appendIntToLCD(PORTBbits.RB0);
      appendIntToLCD(PORTBbits.RB1);
      if (PORTBbits.RB1) break;
    }

  while (TRUE)
  {
    if (PORTBbits.RB0) PORTDbits.RD6 = 1; else PORTDbits.RD6 = 0;
    if (PORTBbits.RB1) PORTDbits.RD7 = 1; else PORTDbits.RD7 = 0;

    PORTDbits.RD0 = 1;
    //PORTAbits.RA4 = 1;

    if (PORTBbits.RB0) PORTDbits.RD6 = 1; else PORTDbits.RD6 = 0;
    if (PORTBbits.RB1) PORTDbits.RD7 = 1; else PORTDbits.RD7 = 0;
    pt = g[5];
    SetDCPWM2(pt);

    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
    PORTDbits.RD0 = 0;
    //PORTAbits.RA4 = 0;
    if (PORTBbits.RB0) PORTDbits.RD6 = 1; else PORTDbits.RD6 = 0;
    if (PORTBbits.RB1) PORTDbits.RD7 = 1; else PORTDbits.RD7 = 0;
    pt = g[63];
    SetDCPWM2(pt);
    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
    delay_ms(100);
  }
  
  */

}

#ifndef DWENGO
void delay_ms(int d)
{
  int i;
  for (i = 0; i < 20; i++) Delay100TCYx(d);
}


#endif
